import ICAL from 'ical.js';
import fs from 'fs';
import path from 'path';
import axios from 'axios';

export const MERGED_CALENDARS_DIR = path.join(process.cwd(), 'calendar');

// Ensure the merged calendars directory exists
fs.mkdirSync(MERGED_CALENDARS_DIR, { recursive: true });

// Utility to sanitize filenames
export const sanitizeFilename = (filename) => filename.replace(/[<>:"/\\|?* ]/g, '_');

// Fetch calendar data from URL or file
export async function fetchCalendarData(calendar) {
    const isFilePath = !calendar.url.startsWith('http');
    try {
        if (isFilePath) {
            return { data: fs.readFileSync(path.resolve(calendar.url), 'utf-8'), ...calendar };
        } else {
            const response = await axios.get(calendar.url);
            return { data: response.data, ...calendar };
        }
    } catch (error) {
        throw new Error(`Error retrieving calendar from ${calendar.url}: ${error.message}`);
    }
}

// Create a top-level VCALENDAR component
export function createCalendarComponent(name) {
    const calendarComponent = new ICAL.Component(['vcalendar', [], []]);
    calendarComponent.updatePropertyWithValue('prodid', '-//Your Product ID//EN');
    calendarComponent.updatePropertyWithValue('version', '2.0');
    calendarComponent.updatePropertyWithValue('name', name);
    return calendarComponent;
}
// Add timezone information
export function addTimezoneComponent(calendarComponent, timezoneId) {
    const timezoneComponent = new ICAL.Component('vtimezone');
    timezoneComponent.updatePropertyWithValue('tzid', timezoneId);
    calendarComponent.addSubcomponent(timezoneComponent);
}

// Add events to the calendar component
export function addEventsToCalendar(calendarComponent, results) {
    results.forEach((result) => {
        const parsed = ICAL.parse(result.data);
        const component = new ICAL.Component(parsed);

        component.getAllSubcomponents('vevent').forEach((event) => {
            const vevent = new ICAL.Event(event);
            const newEvent = new ICAL.Component('vevent');

            // Define start and end times using original timezone info
            const startDate = vevent.startDate; // Keep the original startDate with timezone
            const endDate = vevent.endDate; // Keep the original endDate with timezone

            // Specify time zone ID for DTSTART and DTEND
            const startProp = new ICAL.Property('dtstart');
            startProp.setValue(startDate); // Use the original startDate directly
            startProp.setParameter('tzid', startDate.zone.tzid); // Set the timezone ID

            const endProp = new ICAL.Property('dtend');
            endProp.setValue(endDate); // Use the original endDate directly
            endProp.setParameter('tzid', endDate.zone.tzid); // Set the timezone ID

            // Set properties for newEvent
            newEvent.addProperty(startProp);
            newEvent.addProperty(endProp);
            newEvent.updatePropertyWithValue('uid', vevent.uid);
            newEvent.updatePropertyWithValue('summary', `${result.prefix} ${vevent.summary}`);


            calendarComponent.addSubcomponent(newEvent);
        });
    });
}

// Save calendar data to file
export function saveCalendarFile(filename, content) {
    const filePath = path.join(MERGED_CALENDARS_DIR, filename);
    console.log(`Saving calendar data to file: ${filePath}`);
    fs.writeFileSync(filePath, content);
    return filePath;
}
